@@ -25,7 +25,8 @@ my %SKIP_TEST = (
     '602-mouse-tiny.t'    => "Moose doesn't support ::Tiny",
     '603-mouse-pureperl.t'=> "Moose doesn't have ::PurePerl",
 
-    '031_roles_applied_in_create.t' => 't/lib/* classes are not Moose classes/roles',
+    '031_roles_applied_in_create.t' => 't/lib/*.pm are not for Moose',
+    '013_metaclass_traits.t'        => 't/lib/*.pm are not for Moose',
 );
 
 my @compat_tests;
@@ -65,6 +66,7 @@ File::Find::find(
                 };
                 $src =~ s/Mouse::(?:Util::)?is_class_loaded/Class::MOP::is_class_loaded/g;
                 $src =~ s/Mouse::(?:Util::)?load_class/Class::MOP::load_class/g;
+                $src =~ s/Mouse::Util::class_of/Class::MOP::class_of/g;
                 $src =~ s/Mouse/Moose/g;
                 $src;
             };
